\name{spiral.heatmap}
\alias{spiral.heatmap}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Function to create a spiral heatmap for a time series with seasonality
}
\description{
creates a spiral heatmap for a time series with seasonality. x/y positions are optionally stored in support of post-hoc annotation.
}
\usage{
spiral.heatmap(period, season, x, palette = blues9, 
   bin.type="q", na.col = "yellow", start = 0, 
   clockwise = FALSE, eye.rad = 0.5, bg.col = par("bg"), narc = 21, 
   labels = NULL, label.pos = NULL, 
   label.col = par("fg"), label.cex = 0.8, 
   legend = TRUE, legend.title = "Legend", 
   legend.col = par("fg"), legend.cex = 0.8, 
   legend.labels = NULL, legend.pos = "r",
   xlim=c(-1.3,ifelse(legend && legend.pos=="r", 2, 1.3)),
   ylim=c(ifelse(legend && legend.pos=="b", -2, -1.3), 1.3), 
   store=FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{period}{
    numeric. The period identifier, e.g. year. Must have the same length as \code{season}.
}
  \item{season}{
    the season identifier, e.g. month. May be numeric, character or factor. Must have the same length as \code{period}. Period/season combinations must be sorted in increasing order.
}
  \item{x}{
    the variable to be visualized by heatmap colours (numeric or factor);\cr 
    if numeric, a factor is created based on quantiles, such that each color from \code{palette} is used.\cr
    Must have the same length as \code{period} and \code{season} (missing values are permitted).
    Arbitrary self-chosen binnings of quantitative variables can be achieved by using a factor.
}
  \item{palette}{
    a vector of colors; use a sequential color palette for quantitative or ordinal \code{x}; the default \code{blues9} is not ideal, packages like \pkg{RcolorBrewer} or \pkg{pals} have better palettes on offer.
}
  \item{bin.type}{
    \code{"q"} for quantile bins or \code{"e"} for equally-spaced bins; the number of bins is the number of colors in \code{palette}; note that quantile bins may fail for too many bins because of ties in the data.\cr
    If \code{x} is a factor and \code{"e"} is chosen, it is the user's responsibility that the bins are indeed equally-spaced, and labels for the scale ends must be provided with argument \code{legend.labels} (should be the minimum and maximum of an underlying continuous variable).
}
  \item{na.col}{
    color to use for missing values
}
  \item{start}{
    the angle for the first season (in degrees); the default 0 is at three o'clock.
}
  \item{clockwise}{
    logical; does the plotting occur clockwise or anti-clockwise (default)? 
}
  \item{eye.rad}{
    radius of the empty area (the eye) of the circle at its smallest extent; 
    the total radius is at most 1, i.e. \code{eye.rad} is approximately the proportion of the diameter that is consumed by the eye.
}
  \item{bg.col}{
    background color of the circle (shows in the eye and in the margin); for the entire figure background, modify \code{par("bg")} instead
}
  \item{narc}{
    number of plot points for the arc in each segment, larger number may make the drawing smoother
}
  \item{labels}{
    if \code{NULL}, \code{season} values are shown outside the circle at the center angles of the season's segments. 
    For suppressing these, provide length zero \code{labels} (e.g. \code{labels=character(0)}) with length zero \code{label.pos} (\code{label.pos=numeric(0)}). 
}
  \item{label.pos}{
    radial position of labels, specified in degrees. Must have same length as \code{labels}.
}
  \item{label.col}{
    color for label text
}
  \item{label.cex}{
    character size for label text
}
  \item{legend}{
    logical: Should a legend be printed? If the automatic legend is suppressed, you can add one yourself. 
    You can provide space for it by adjusting the choice of \code{xlim} or \code{ylim} to your needs.
}
  \item{legend.title}{
    title for the legend
}
  \item{legend.col}{
    color of legend text
}
  \item{legend.cex}{
    size of legend text
}
  \item{legend.labels}{
    labels for legend scale ends; only used for \code{bin.type="e"} with factor.valued \code{x}
}
  \item{legend.pos}{
    position of legend, can be right (\code{"r"}) or bottom (\code{"b"}); ignored for \code{bin.type="q"}
}
  \item{xlim}{
    the default depends on whether or not a legend on the right is requested: it is \code{-1.3, 1.3} without a legend on the right, and 
    \code{xlim[2]} becomes 2 with a legend on the right.
}
  \item{ylim}{
    the default depends on whether or not a legend on the right is requested: it is \code{c(-1.3, 1.3)} without a legend at the bottom, and \code{ylim[1]} becomes -2 with a legend at the bottom. 
}
  \item{store}{
    logical (default: FALSE): should a data frame with the x/y positions of polygon boundaries (\code{fromx}, \code{fromy},\code{midx}, \code{midy}, \code{tox}, \code{toy}) be stored? If so, an element \code{points.xy} is added to the output list and can be used for subsequently adding markers.\cr For the mid position to be exactly in the middle, choose \code{narc} as an odd number.
}
  \item{\dots}{
    further arguments to be given to a \code{text} or \code{cut} call (note: there may be warnings from the respective other in case of invalid parameters, like \code{dig.lab} for \code{cut}, which is invalid for \code{text}; these can be ignored)
}
}
\details{
The function works for a completely regular time series with time points sorted in increasing order. Each period, except for the first and last, 
must have a complete sequence of all seasons. If some values of \code{x} are not available, observations with missing values must be included.

The center of the spiral is at \code{c(0,0)}, and a circle with radius \code{eye.rad} is kept clear. The spiral starts at the inside and moves outward with time,  and it ends at radius at most 1 from the center (if both the first and the last period have observations for all seasons).

For each period, the middle of the first season's segment is at the \code{start} angle. 
Polygons for all periods have the same extent in radius direction, but arc lengths of newer season segments are of course larger.

For displaying a time series of quantitative data (which is the function's main purpose), a sequential color palette should usually be chosen. If a palette with many colors is chosen (e.g. 50), it is presumably most adequate to use \code{bin.type="e"}. If that type is used with a factor-valued \code{x} (from a pre-binned quantitative variable), \code{legend.labels} must be specified for the ends of the color legend.

It may sometimes be advisable to manually create a factor-valued \code{x}, in order to allow to bin the data in a desired way instead of using the default bin types (but some adjustments are also possible via missing value colors, see Examples section).

It is also conceivable to use the function for showing the development of exclusive states over time (like in Andrew Elliott's use case that inspired 
the creation of this function, see reference).
}
\value{
The function returns a list with the four to six elements 

  \item{period.range }{the range of the period values.}
  \item{rad.range }{the range of the radius values.}
  \item{angle}{a named numeric vector with angles in degrees for the middle of season segments, named with season names. }
  \item{heatcolors}{a vector of the colors used for mapping, named with the mapped factor levels.}
  \item{xrange}{for numeric \code{x} only: the range of the \code{x} values}
  \item{points.xy}{if \code{store=TRUE}, this is a \code{data.frame} with columns \code{period}, \code{season}, \code{fromx}, \code{fromy}, \code{midx}, \code{midy}, \code{tox}, \code{toy} }
}
\references{
Inspired by Andrew Elliots visualization of his baby daughter's sleep patterns at 
\url{https://www.reddit.com/r/dataisbeautiful/comments/5l39mu/my_daughters_sleeping_patterns_for_the_first_4/}.
}
\author{
Ulrike Groemping
}
\note{
Note that \code{main} and \code{sub} do not work for this function. Look at the last two examples for placing custom annotation.

Note that several R packages provide very good color palettes, e.g. \pkg{RColorBrewer} or \pkg{pals}. These are not used in examples in order to avoid package dependencies.

Furthermore, note that edges look much smoother on cairo type devices, e.g. \code{cairo_pdf} or \code{png} with type \code{cairo}.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
mdeaths
## mdeaths is a time series object
## preparing it for plotting
year <- rep(1974:1979, each=12)
month <- c("Jan",  "Feb",  "Mar",  "Apr",  "May",  "Jun",
           "Jul",  "Aug",  "Sep",  "Oct",  "Nov",  "Dec")
month <- factor(rep(month, 6), levels=month)

## with default legend
spiral.heatmap(year, month, mdeaths)

## improved legend by allowing 4 digits to function cut
suppressWarnings({
mysph <- spiral.heatmap(year, month, mdeaths, dig.lab=4)})

## custom legend using the output from the function
## (not good, just for demonstration)
suppressWarnings({
mysph2 <- spiral.heatmap(year, month, mdeaths, 
    legend=FALSE, ylim=c(-2, 1.3), dig.lab=4)
    })
legend("bottom", ncol=3, legend=names(mysph2$heatcolors), 
    lty=1, lwd=10, col=mysph2$heatcolors, border=mysph2$heatcolors, 
    cex=0.8, bty="n")

## equally-spaced bins
mysph3 <- spiral.heatmap(year, month, mdeaths, bin.type="e")

## finer palette
mysph4 <- spiral.heatmap(year, month, mdeaths, 
    bin.type="e", palette=rev(grey.colors(50)))

## isolating an outlying value 
## for finer resolution in the rest
mymdeaths <- mdeaths
mymdeaths[mymdeaths==2750] <- NA
mysph4 <- spiral.heatmap(year, month, mymdeaths, 
    bin.type="e", palette=rev(grey.colors(50)), na.col="black")
title(main="Spiral heatmap with outlying number 2750 shown in black")
## title places text at the very top
mtext(side=3, line=-1.5,
'achieved by setting this number to NA before calling spiral.heatmap with na.col="black"')
## text or mtext can be used for placing text where one wants to

## placing the legend at the bottom instead
## and using stored info for placing a marker at a position of interest
mysph5 <- spiral.heatmap(year, month, mymdeaths, 
    bin.type="e", palette=rev(grey.colors(50)), na.col="black",
    legend.pos="b", store=TRUE)
mtext(side=1, line=-2, cex=0.8, "black: 2750")
mtext(side=3, line=-1, font=2, cex=1.2, 
  "Spiral heatmap with outlying number 2750 shown in black", adj=0)
  rownum <- which(mysph5$points.xy$period==1978 & mysph5$points.xy$season=="Aug")
  points(mysph5$points.xy$midx[rownum], mysph5$points.xy$midy[rownum], pch=16, col="red")
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
\keyword{ hplot }
\keyword{ ts }